Ext.namespace('Zarafa.plugins.files.backend.Owncloud.data.singleton');

/**
 * @class Zarafa.plugins.files.backend.Owncloud.data.singleton.ShareStore
 * @extends Object
 *
 * This singleton provides access to the {@link Zarafa.plugins.files.backend.Owncloud.data.ShareGridStore ShareGridStore}.
 * It must be initialized once by calling the init method.
 */
Zarafa.plugins.files.backend.Owncloud.data.singleton.ShareStore = Ext.extend(Object, {

	/**
	 * @property
	 * @type Zarafa.plugins.files.data.AccountStore
	 * @private
	 */
	store: undefined,

	/**
	 * Triggers a call to the backend to load version information.
	 */
	init: function () {
		this.store = new Zarafa.plugins.files.backend.Owncloud.data.ShareGridStore();
	},

	/**
	 * Loads userdata to the store.
	 *
	 * @param shareid
	 * @param share_with
	 * @param permissions
	 */
	addUser: function (shareid, share_with, permissions) {
		var permission_create = false;
		var permission_change = false;
		var permission_delete = false;
		var permission_share = false;

		// parse the permission number
		if ((permissions - 16) >= 1) {
			permission_share = true;
			permissions -= 16;
		}
		if ((permissions - 8) >= 1) {
			permission_delete = true;
			permissions -= 8;
		}
		if ((permissions - 4) >= 1) {
			permission_create = true;
			permissions -= 4;
		}
		if ((permissions - 2) >= 1) {
			permission_change = true;
		}

		var record = [shareid, share_with, "user", permission_create, permission_change, permission_delete, permission_share];

		this.store.loadData([record], true);
	},

	/**
	 * Loads groupdata to the store.
	 *
	 * @param shareid
	 * @param share_with
	 * @param permissions
	 */
	addGroup: function (shareid, share_with, permissions) {
		var permission_create = false;
		var permission_change = false;
		var permission_delete = false;
		var permission_share = false;

		// parse the permission number
		if ((permissions - 16) >= 1) {
			permission_share = true;
			permissions -= 16;
		}
		if ((permissions - 8) >= 1) {
			permission_delete = true;
			permissions -= 8;
		}
		if ((permissions - 4) >= 1) {
			permission_create = true;
			permissions -= 4;
		}
		if ((permissions - 2) >= 1) {
			permission_change = true;
		}

		var record = [shareid, share_with, "group", permission_create, permission_change, permission_delete, permission_share];

		this.store.loadData([record], true);
	},

	/**
	 * Get instance of the {@link Zarafa.plugins.files.data.AccountStore Accountstore}
	 * @return {Zarafa.plugins.files.data.AccountStore}
	 */
	getStore: function () {
		return this.store;
	}
});

// Make it a Singleton
Zarafa.plugins.files.backend.Owncloud.data.singleton.ShareStore = new Zarafa.plugins.files.backend.Owncloud.data.singleton.ShareStore();