Ext.namespace('Zarafa.plugins.files.backend.Owncloud.ui');

/**
 * @class Zarafa.plugins.files.backend.Owncloud.ui.FilesShareDialogPanel
 * @extends Zarafa.plugins.files.ui.dialogs.SharePanel
 * @xtype filesplugin.owncloud.filessharedialogpanel
 *
 * The panel contains all logic and UI elements that are needed for the OCS sharing functionality.
 */
Zarafa.plugins.files.backend.Owncloud.ui.FilesShareDialogPanel = Ext.extend(Zarafa.plugins.files.ui.dialogs.SharePanel, {
	/**
	 * @var Ext.LoadMask loading mask
	 */
	loadMask: undefined,

	/**
	 * @var bool Flag for the password field.
	 */
	passwordChanged: false,

	/**
	 * @var bool Flag for the date field.
	 */
	expirationDateChanged: false,

	/**
	 * @var bool Flag for the edit checkbox.
	 */
	pubUploadChanged: false,

	/**
	 * @var integer The id of the linkshare, -1 if no linkshare is set.
	 */
	linkShareID: -1,

	/**
	 * Constructor - init store and UI
	 *
	 * @constructor
	 * @param config
	 */
	constructor: function (config) {
		config = config || {};

		Zarafa.plugins.files.backend.Owncloud.data.singleton.ShareStore.init();
		this.setupGridStoreListeners();

		Ext.applyIf(config, {
			listeners: {
				afterrender: this.checkSharedRecord
			},
			height   : 450,
			items    : [{
				xtype     : "fieldset",
				title     : dgettext("plugin_files_owncloud", "Share with user/group"),
				autoHeight: true,
				ref       : "userfieldset",
				items     : [{
					xtype : "filesplugin.owncloud.filesshareusergrid",
					ref   : "../usergrid",
					height: 200
				}]
			}, {
				xtype     : "checkbox",
				fieldLabel: "",
				boxLabel  : dgettext("plugin_files_owncloud", "Share via link"),
				ref       : "linkcheckbox",
				inputValue: "sharelink",
				style: {
					marginLeft: '6px'
				},
				listeners : {
					check: this.onShareViaLinkChecked.createDelegate(this)
				}
			}, {
				xtype     : "fieldset",
				title     : dgettext("plugin_files_owncloud", "Share via link"),
				autoHeight: true,
				ref       : "linkfieldset",
				hidden    : true,
				items     : [{
					layout  : "column",
					border  : false,
					defaults: {
						border: false
					},
					anchor  : "0",
					items   : [{
						columnWidth: .95,
						layout     : "form",
						items      : {
							xtype        : "textfield",
							fieldLabel   : dgettext("plugin_files_owncloud", "Public link"),
							ref          : "../../../linkfield",
							anchor       : '100%',
							selectOnFocus: true,
							readOnly     : true
						}
					}, {
						columnWidth: .05,
						items      : {
							xtype       : "copybutton",
							swfPath     : "plugins/filesbackendOwncloud/resources/flash/ZeroClipboard.swf",
							completeText: dgettext("plugin_files_owncloud", "Link copied to clipboard"),
							iconCls     : "icon_copy_clipboard",
							getValue    : this.getPublicLinkValue.createDelegate(this)
						}
					}]
				}, {
					xtype     : "checkbox",
					fieldLabel: dgettext("plugin_files_owncloud", "Password protect"),
					boxLabel  : "",
					ref       : "../passwordcheckbox",
					inputValue: "pwprotected",
					listeners : {
						check: this.onUsePasswordChecked.createDelegate(this)
					}
				}, {
					xtype     : "textfield",
					fieldLabel: dgettext("plugin_files_owncloud", "Password"),
					ref       : "../passwordfield",
					hidden    : true,
					name      : "textvalue",
					listeners : {
						change: this.onPasswordChange.createDelegate(this),
						keyup : this.onPasswordChange.createDelegate(this)
					}
				}, {
					xtype     : "checkbox",
					fieldLabel: dgettext("plugin_files_owncloud", "Public upload"),
					boxLabel  : "",
					hidden    : true,
					ref       : "../editcheckbox",
					inputValue: "allowediting",
					listeners : {
						check: this.onAllowEditingChecked.createDelegate(this)
					}
				}, {
					xtype     : "checkbox",
					fieldLabel: dgettext("plugin_files_owncloud", "Expiration date"),
					boxLabel  : "",
					ref       : "../expirationcheckbox",
					inputValue: "useexpiration",
					listeners : {
						check: this.onUseExpirationDateChecked.createDelegate(this)
					}
				}, {
					xtype     : "datefield",
					ref       : "../expirationfield",
					hidden    : true,
					fieldLabel: dgettext("plugin_files_owncloud", "Date"),
					minValue  : new Date(new Date().getTime() + 24 * 60 * 60 * 1000), // tomorrow
					width     : 170,
					format    : 'Y-m-d',
					listeners : {
						change: this.onExpirationDateChange.createDelegate(this),
						keyup : this.onExpirationDateChange.createDelegate(this)
					}
				}]
			}],
			buttons  : [{
				xtype   : 'button',
				text    : dgettext('plugin_files_owncloud', 'Unshare'),
				tooltip : {
					title: dgettext('plugin_files_owncloud', 'Completely unshare this record.'),
					text : dgettext('plugin_files_owncloud', 'Remove all shares.')
				},
				handler : this.removeAllShares,
				ref     : "../unshareButton",
				disabled: true,
				scope   : this
			}, {
				xtype  : 'button',
				text   : '&nbsp;&nbsp;&nbsp;&nbsp;' + dgettext('plugin_files_owncloud', 'Done'),
				tooltip: {
					title: dgettext('plugin_files_owncloud', 'Share in ownCloud'),
					text : dgettext('plugin_files_owncloud', 'Share in ownCloud.')
				},
				iconCls: 'files_icon_16_share',
				handler: this.onDoneButtonClick,
				scope  : this
			}]
		});

		Zarafa.plugins.files.backend.Owncloud.ui.FilesShareDialogPanel.superclass.constructor.call(this, config);
	},

	/**
	 * Create the onUpdate and onRemove listeners for the {@link Zarafa.plugins.files.backend.Owncloud.data.ShareGridStore ShareGridStore}.
	 * @private
	 */
	setupGridStoreListeners: function () {
		var store = Zarafa.plugins.files.backend.Owncloud.data.singleton.ShareStore.getStore();

		store.on("update", this.onGridStoreUpdate, this);
		store.on("remove", this.onGridStoreRemove, this);
	},

	/**
	 * Eventhandler for the remove event of the {@link Zarafa.plugins.files.backend.Owncloud.data.ShareGridStore ShareGridStore}.
	 * If the shareid is set, it will remove the share from the backend.
	 *
	 * @param store
	 * @param record
	 * @param index
	 * @private
	 */
	onGridStoreRemove: function (store, record, index) {
		// check if an id is set - if so, remove the old share
		if (record.get("id") != "") {
			this.removeShareByID(record.get("id"));
		}
	},

	/**
	 * Eventhandler for the update event of the {@link Zarafa.plugins.files.backend.Owncloud.data.ShareGridStore ShareGridStore}.
	 * This will first remove the new dirty record and then create a new one. So the save function of the store is not used :)
	 *
	 * @param store
	 * @param record
	 * @param operation
	 * @private
	 */
	onGridStoreUpdate: function (store, record, operation) {
		store.remove(record); // remove the dirty record

		var permissions = 1;
		if (record.get("permission_change")) {
			permissions += 2;
		}
		if (record.get("permission_create")) {
			permissions += 4;
		}
		if (record.get("permission_delete")) {
			permissions += 8;
		}
		if (record.get("permission_share")) {
			permissions += 16;
		}

		// readd it
		if (record.get("type") === "group") {
			this.createShareByGroup(record.get("share_with"), permissions);
		} else {
			this.createShareByUser(record.get("share_with"), permissions);
		}
	},

	/**
	 * Eventhandler for the checkbox change event.
	 *
	 * @param checkbox
	 * @param checked
	 * @private
	 */
	onShareViaLinkChecked: function (checkbox, checked) {
		if (checked) {
			this.linkfieldset.show();

			// create a new share
			this.createShareByLink();
		} else {
			this.linkfieldset.hide();
			this.removeShareByID(this.linkShareID);
			this.linkShareID = -1; // reset the id
		}
	},

	/**
	 * Eventhandler for the checkbox change event.
	 *
	 * @param checkbox
	 * @param checked
	 * @private
	 */
	onUsePasswordChecked: function (checkbox, checked) {
		if (checked) {
			this.passwordfield.show();
		} else {
			this.passwordfield.hide();
		}
	},

	/**
	 * Eventhandler for the textfield change event.
	 *
	 * @param field
	 * @param event
	 * @private
	 */
	onPasswordChange: function (field, event) {
		this.passwordChanged = true;
	},

	/**
	 * Eventhandler for the checkbox change event.
	 *
	 * @param checkbox
	 * @param checked
	 * @private
	 */
	onAllowEditingChecked: function (checkbox, checked) {
		this.pubUploadChanged = true;
	},

	/**
	 * Eventhandler for the checkbox change event.
	 *
	 * @param checkbox
	 * @param checked
	 * @private
	 */
	onUseExpirationDateChecked: function (checkbox, checked) {
		if (checked) {
			this.expirationfield.show();
		} else {
			this.expirationfield.hide();
		}
	},

	/**
	 * Eventhandler for the datefield change event.
	 *
	 * @param field
	 * @param event
	 * @private
	 */
	onExpirationDateChange: function (field, event) {
		this.expirationDateChanged = true;
	},

	/**
	 * Return the current value of the public link textfield.
	 *
	 * @return {*|String}
	 */
	getPublicLinkValue: function () {
		return this.linkfield.getValue();
	},

	/**
	 * Close the dialog and clean all eventhandlers.
	 * @private
	 */
	closeDialog: function () {
		var store = Zarafa.plugins.files.backend.Owncloud.data.singleton.ShareStore.getStore();

		store.un("update", this.onGridStoreUpdate, this);
		store.un("remove", this.onGridStoreRemove, this);

		this.dialog.close();
	},

	/**
	 * Close the loadmask
	 * @private
	 */
	closeLoadMask: function () {
		this.loadMask.hide();
	},

	/**
	 * Eventhandler for the "done" button.
	 * It will save all changes of the linkshare and close the dialog.
	 * @private
	 */
	onDoneButtonClick: function () {

		// check if we have a link or user/group share
		if (this.linkcheckbox.getValue()) { // we have a link share
			// check if we have to update the share
			this.updateShareByLink();
		} else { // we have a user/group share
			this.closeDialog();
		}
	},

	/**
	 * This method checks the dialog records for existing shares. If shares were found, it will try to load the details.
	 * @private
	 */
	checkSharedRecord: function () {
		// init loading mask after the panel was rendered
		this.loadMask = new Ext.LoadMask(this.getEl(), {msg: dgettext("plugin_files_owncloud", "Loading details...")});

		// check if we have a shared record where we should load details
		if (Ext.isDefined(this.ownerCt.records) && this.ownerCt.records.length == 1) {
			var record = this.ownerCt.records[0];

			// enable the edit checkbox if we have a folder record
			if (record.get("type") === Zarafa.plugins.files.data.FileTypes.FOLDER) {
				this.editcheckbox.show();
			}

			if (record.get("isshared") === true) {
				this.initSharedRecord();
			}
		}
	},

	/**
	 * This method requests the sharedetails from the backend.
	 * @private
	 */
	initSharedRecord: function () {
		this.loadMask.show();

		var recIds = new Array(this.ownerCt.records[0].get("id"));

		container.getRequest().singleRequest(
			'filesbrowsermodule',
			'loadsharingdetails',
			{
				records: recIds
			},
			new Zarafa.plugins.files.backend.Owncloud.data.ResponseHandler({
				successCallback: this.initGuiFromSharedRecord.createDelegate(this)
			})
		);
	},

	/**
	 * Callback for the loadsharingdetails response. This function will initialize the UI with the given
	 * share records.
	 *
	 * @param response
	 * @private
	 */
	initGuiFromSharedRecord: function (response) {
		var shares = response.shares[this.ownerCt.records[0].get("id")];
		var shareid = undefined;

		for (shareid in shares) {
			var share = shares[shareid];

			// check sharing type: ‘0’ = user; ‘1’ = group; ‘3’ = public link
			var shareType = parseInt(share["share_type"]);

			if (shareType == 3) { // public link
				// store the id of this share
				this.linkShareID = shareid;

				// change gui
				this.linkfieldset.show();
				this.linkcheckbox.suspendEvents(false); // Stop all events.
				this.linkcheckbox.setValue(true); // check checkbox
				this.linkcheckbox.resumeEvents(); // resume events
				this.linkfield.setValue(share["url"]);

				// check expiration
				if (!Ext.isEmpty(share["expiration"])) {
					this.expirationcheckbox.setValue(true); // check checkbox
					var dt = new Date(share["expiration"]);
					this.expirationfield.setValue(dt);
				}
				// check password
				if (!Ext.isEmpty(share["share_with"])) {
					this.passwordcheckbox.setValue(true); // check checkbox
					this.passwordfield.setValue("******");
				}
				//check permissions
				if (!Ext.isEmpty(share["permissions"])) {
					if (parseInt(share["permissions"]) == 7) {
						this.editcheckbox.suspendEvents(false); // Stop all events.
						this.editcheckbox.setValue(true); // check checkbox
						this.editcheckbox.resumeEvents(); // resume events
					}
				}
			} else if (shareType == 1) { // group
				Zarafa.plugins.files.backend.Owncloud.data.singleton.ShareStore.addGroup(shareid, share["share_with"], share["permissions"]);
			} else { // user
				Zarafa.plugins.files.backend.Owncloud.data.singleton.ShareStore.addUser(shareid, share["share_with"], share["permissions"]);
			}

			this.unshareButton.setDisabled(false);
		}

		this.loadMask.hide();
	},

	/**
	 * This method will request the creation of a new linkshare from the backend.
	 * @private
	 */
	createShareByLink: function () {
		this.loadMask.show();

		var recIds = new Array(this.ownerCt.records[0].get("id")); // we're only going to share one record

		var shareOpts = {
			shareType: 3
		};

		container.getRequest().singleRequest(
			'filesbrowsermodule',
			'createnewshare',
			{
				records: recIds,
				options: shareOpts
			},
			new Zarafa.plugins.files.backend.Owncloud.data.ResponseHandler({
				successCallback: this.shareByLinkCreated.createDelegate(this),
				failureCallback: this.closeDialog.createDelegate(this)
			})
		);
	},

	/**
	 * Callback for the createnewshare response. It will update the parent record and the UI.
	 *
	 * @param response
	 * @private
	 */
	shareByLinkCreated: function (response) {
		var share = response.shares[this.ownerCt.records[0].get("id").replace(/\/+$/g, "")];
		this.linkfield.setValue(share["url"]);

		// store the id of this share
		this.linkShareID = share["id"];

		var recIds = this.ownerCt.records[0].get("sharedid") || [];
		recIds.push(share["id"]);

		// also update the parent record
		this.ownerCt.records[0].set("sharedid", recIds);
		this.ownerCt.records[0].set("isshared", true);

		// enable the unshare button
		this.unshareButton.setDisabled(false);

		this.loadMask.hide();
	},

	/**
	 * This method will request the deletion of a all shares from the backend.
	 * @private
	 */
	removeAllShares: function () {
		this.loadMask.show();

		var recIds = this.ownerCt.records[0].get("sharedid");
		var accId = this.ownerCt.records[0].getAccount().get("id");

		if (recIds.length < 1) {
			this.closeDialog();
		} else {
			container.getRequest().singleRequest(
				'filesbrowsermodule',
				'deleteexistingshare',
				{
					records  : recIds,
					accountid: accId
				},
				new Zarafa.plugins.files.backend.Owncloud.data.ResponseHandler({
					successCallback: this.allSharesRemoved.createDelegate(this),
					failureCallback: this.closeDialog.createDelegate(this)
				})
			);
		}
	},

	/**
	 * Callback for the deleteexistingshare response. It will update the parent record and set it to unshared.
	 * The dialog will be closed.
	 *
	 * @param response
	 * @private
	 */
	allSharesRemoved: function (response) {
		// also update the parent record
		this.ownerCt.records[0].set("sharedid", []);
		this.ownerCt.records[0].set("isshared", false);

		this.loadMask.hide();

		this.closeDialog();
	},

	/**
	 * This method will request the deletion of a one share from the backend.
	 * @private
	 */
	removeShareByID: function (id) {
		var accId = this.ownerCt.records[0].getAccount().get("id");

		container.getRequest().singleRequest(
			'filesbrowsermodule',
			'deleteexistingshare',
			{
				records  : [id],
				accountid: accId
			},
			new Zarafa.plugins.files.backend.Owncloud.data.ResponseHandler({
				successCallback: this.shareByIDRemoved.createDelegate(this, [id], true),
				failureCallback: this.closeDialog.createDelegate(this)
			})
		);
	},

	/**
	 * Callback for the deleteexistingshare response. It will update the parent record and the UI.
	 *
	 * @param response
	 * @param id
	 * @private
	 */
	shareByIDRemoved: function (response, id) {
		var recIds = this.ownerCt.records[0].get("sharedid") || [];
		var index = recIds.indexOf(parseInt(id));

		if (index > -1) {
			recIds.splice(index, 1); // remove the id from the array
		}

		// also update the parent record
		this.ownerCt.records[0].set("sharedid", recIds);

		if (recIds.length == 0) {
			this.ownerCt.records[0].set("isshared", false);
		}
	},

	/**
	 * This method will request the creation of a new user share from the backend.
	 *
	 * @param share_with Username to share this record with.
	 * @param permissions Integer specifying the OCS permissions
	 * @private
	 */
	createShareByUser: function (share_with, permissions) {
		this.loadMask.show();

		var recIds = new Array(this.ownerCt.records[0].get("id")); // we're only going to share one record

		var shareOpts = {
			shareType  : 0,
			shareWith  : share_with,
			permissions: permissions
		};

		container.getRequest().singleRequest(
			'filesbrowsermodule',
			'createnewshare',
			{
				records: recIds,
				options: shareOpts
			},
			new Zarafa.plugins.files.backend.Owncloud.data.ResponseHandler({
				successCallback: this.shareByUserCreated.createDelegate(this, [share_with, permissions], true),
				failureCallback: this.closeLoadMask.createDelegate(this)
			})
		);
	},

	/**
	 * Callback for the createnewshare response. It will update the parent record and the UI.
	 * This method will add the created usershare to the share grid.
	 *
	 * @param response
	 * @param share_with
	 * @param permissions
	 * @private
	 */
	shareByUserCreated: function (response, share_with, permissions) {
		var share = response.shares[this.ownerCt.records[0].get("id").replace(/\/+$/g, "")];

		var recIds = this.ownerCt.records[0].get("sharedid") || [];
		recIds.push(share["id"]);

		// update the parent record
		this.ownerCt.records[0].set("sharedid", recIds);
		this.ownerCt.records[0].set("isshared", true);

		// enable the unshare button
		this.unshareButton.setDisabled(false);

		Zarafa.plugins.files.backend.Owncloud.data.singleton.ShareStore.addUser(share["id"], share_with, permissions);
		this.loadMask.hide();
	},

	/**
	 * This method will request the creation of a new group share from the backend.
	 *
	 * @param share_with Groupname to share this record with.
	 * @param permissions Integer specifying the OCS permissions
	 * @private
	 */
	createShareByGroup: function (share_with, permissions) {
		this.loadMask.show();

		var recIds = new Array(this.ownerCt.records[0].get("id")); // we're only going to share one record

		var shareOpts = {
			shareType  : 1,
			shareWith  : share_with,
			permissions: permissions
		};

		container.getRequest().singleRequest(
			'filesbrowsermodule',
			'createnewshare',
			{
				records: recIds,
				options: shareOpts
			},
			new Zarafa.plugins.files.backend.Owncloud.data.ResponseHandler({
				successCallback: this.shareByGroupCreated.createDelegate(this, [share_with, permissions], true),
				failureCallback: this.closeLoadMask.createDelegate(this)
			})
		);
	},

	/**
	 * Callback for the createnewshare response. It will update the parent record and the UI.
	 * This method will add the created groupshare to the share grid.
	 *
	 * @param response
	 * @param share_with
	 * @param permissions
	 * @private
	 */
	shareByGroupCreated: function (response, share_with, permissions) {
		var share = response.shares[this.ownerCt.records[0].get("id").replace(/\/+$/g, "")];

		var recIds = this.ownerCt.records[0].get("sharedid") || [];
		recIds.push(share["id"]);

		// update the parent record
		this.ownerCt.records[0].set("sharedid", recIds);
		this.ownerCt.records[0].set("isshared", true);

		// enable the unshare button
		this.unshareButton.setDisabled(false);

		Zarafa.plugins.files.backend.Owncloud.data.singleton.ShareStore.addGroup(share["id"], share_with, permissions);
		this.loadMask.hide();
	},

	/**
	 * This method will request the modification of the lin share options from the backend.
	 *
	 * @private
	 */
	updateShareByLink: function () {
		this.loadMask.show();

		var accId = this.ownerCt.records[0].getAccount().get("id");

		var shareOpts = {};

		if (this.passwordChanged) {
			shareOpts["password"] = this.passwordfield.getValue();
		}
		if (this.pubUploadChanged) {
			// don't use publicUpload as this flag does not work (yet?) - ocs bug

			if (this.editcheckbox.getValue()) {
				shareOpts["permissions"] = 7;
			} else {
				shareOpts["permissions"] = 1;
			}
		}
		if (this.expirationDateChanged) {
			shareOpts["expireDate"] = this.expirationfield.getRawValue();
		}

		container.getRequest().singleRequest(
			'filesbrowsermodule',
			'updateexistingshare',
			{
				records  : [this.linkShareID],
				accountid: accId,
				options  : shareOpts
			},
			new Zarafa.plugins.files.backend.Owncloud.data.ResponseHandler({
				successCallback: this.shareByLinkUpdated.createDelegate(this),
				failureCallback: this.closeLoadMask.createDelegate(this)
			})
		);
	},

	/**
	 * Callback for the updateexistingshare response. This function simply closes the dialog.
	 *
	 * @param response
	 * @private
	 */
	shareByLinkUpdated: function (response) {
		this.loadMask.hide();
		this.closeDialog();
	}
});

Ext.reg('filesplugin.owncloud.filessharedialogpanel', Zarafa.plugins.files.backend.Owncloud.ui.FilesShareDialogPanel);