Ext.namespace('Zarafa.plugins.files.backend.Owncloud.ui');

/**
 * @class Zarafa.plugins.files.backend.Owncloud.ui.FilesShareUserGridColumnModel
 * @extends Zarafa.common.ui.grid.ColumnModel
 *
 * The Column model for the share grid.
 */
Zarafa.plugins.files.backend.Owncloud.ui.FilesShareUserGridColumnModel = Ext.extend(Zarafa.common.ui.grid.ColumnModel, {

	/**
	 * @constructor
	 * @param config Configuration structure
	 */
	constructor: function (config) {
		config = config || {};

		this.defaultColumns = this.createDefaultColumns();

		Ext.applyIf(config, {
			columns : this.defaultColumns,
			defaults: {
				sortable: true
			}
		});
		Ext.apply(this, config);

		Zarafa.plugins.files.backend.Owncloud.ui.FilesShareUserGridColumnModel.superclass.constructor.call(this, config);
	},

	/**
	 * Create an array of {@link Ext.grid.Column columns} which must be visible within
	 * the default view of this {@link Ext.grid.ColumnModel ColumnModel}.
	 *
	 * @return {Ext.grid.Column[]} The array of columns
	 * @private
	 */
	createDefaultColumns: function () {
		return [
			new Ext.grid.RowNumberer(),
			{
				header   : dgettext('plugin_files', 'ID'),
				dataIndex: 'id',
				width    : 50,
				hidden   : true,
				tooltip  : dgettext('plugin_files', 'Sort by: ID')
			}, {
				header   : dgettext('plugin_files', 'Name'),
				dataIndex: 'share_with',
				flex     : 1,
				sortable : true,
				tooltip  : dgettext('plugin_files', 'Sort by: Name'),
				editor   : new Ext.form.TextField({})
			}, {
				header   : dgettext('plugin_files', 'Type'),
				dataIndex: 'type',
				width    : 40,
				sortable : true,
				renderer : this.shareTypeRenderer,
				tooltip  : dgettext('plugin_files', 'Sort by: Type'),
				editor   : {
					xtype     : 'selectbox',
					width     : 100,
					store     : [
						["user", dgettext('plugin_files', 'User')],
						["group", dgettext('plugin_files', 'Group')]
					],
					mode      : 'local',
					allowBlank: false
				}
			}, {
				header   : dgettext('plugin_files', 'Create'),
				dataIndex: 'permission_create',
				width    : 30,
				sortable : false,
				renderer : this.yesNoRenderer,
				editor   : new Ext.form.Checkbox({})
			}, {
				header   : dgettext('plugin_files', 'Change'),
				dataIndex: 'permission_change',
				width    : 30,
				sortable : false,
				renderer : this.yesNoRenderer,
				editor   : new Ext.form.Checkbox({})
			}, {
				header   : dgettext('plugin_files', 'Delete'),
				dataIndex: 'permission_delete',
				width    : 30,
				sortable : false,
				renderer : this.yesNoRenderer,
				editor   : new Ext.form.Checkbox({})
			}, {
				header   : dgettext('plugin_files', 'Share'),
				dataIndex: 'permission_share',
				width    : 30,
				sortable : false,
				renderer : this.yesNoRenderer,
				editor   : new Ext.form.Checkbox({})
			}];
	},

	/**
	 * This renderer will render the type column. It will set the css class either to group or user.
	 *
	 * @param value
	 * @param p
	 * @param record
	 * @return {string}
	 */
	shareTypeRenderer: function (value, p, record) {

		p.css = "shareicon_16_" + value;

		// add extra css class for empty cell
		p.css += ' zarafa-grid-empty-cell';

		return '';
	},

	/**
	 * This renderer will render the boolean columns.
	 * It will show nice icons for true and false.
	 *
	 * @param value
	 * @param p
	 * @param record
	 * @return {string}
	 */
	yesNoRenderer: function (value, p, record) {

		if (value) {
			p.css = "shareicon_16_yes";
		} else {
			p.css = "shareicon_16_no";
		}

		// add extra css class for empty cell
		p.css += ' zarafa-grid-empty-cell';

		return '';
	}
});
